/*
 * slot.c
 *
 * Copyright (C) 2006-2008 Atmark Techno, Inc.	All Rights Reserved.
 *
 */

#include <ctype.h>
/* #include "medium.h" */

typedef unsigned char u8;

#ifdef __PPC__
#define SLOT_BASE_ADDR	0xf0ffd000
#else
#define SLOT_BASE_ADDR	0xffffd000
#endif

/*  slot status	 */
#define SLOT_READY	(0)
#define SLOT_RUN	(1)
#define SLOT_STOP	(2)

/* switch */
#define CHATTERING_CLEAR_CNT (2)
#define SWITCH_NUM	(3)
#define SWITCH(no)	(1 << (no))
#define SWITCH_ALL	(SWITCH(0) | SWITCH(1) | SWITCH(2))
#define READ_SWITCH	(*(volatile u8 *)(SLOT_BASE_ADDR + 0x4))

/* 7seg-led */
#define SEG7_NUM		(3)
#define SEG7(no)		(1 << (no))
#define SEG7_ALL		(SEG7(0) | SEG7(1) | SEG7(2))
#define WRITE_SEG7(no,v)	(*(volatile u8 *)(SLOT_BASE_ADDR + (no)) = (v))

/* led */
#define WRITE_LEDS(v)	(*(volatile u8 *)(SLOT_BASE_ADDR + 0x3) = (v))

/* rotary switch */
#define READ_RSWITCH	(*(volatile u8 *)(SLOT_BASE_ADDR + 0x5))

static u8 clear_switch_chattering(void)
{
	static u8 count[SWITCH_NUM];
	static u8 status = 0;
	u8	      reg_sw = READ_SWITCH;
	int	      i;

	for (i = 0; i < SWITCH_NUM; i++) {
		if (reg_sw & SWITCH(i)) {
			if (count[i] < CHATTERING_CLEAR_CNT)
				count[i]++;
		} else {
			if (count[i] > 0)
				count[i]--;
		}
		
		if (count[i] == CHATTERING_CLEAR_CNT)
			status |=  SWITCH(i);
		else if (count[i] == 0)
			status &= ~SWITCH(i);
	}
	
	return status;
}

static void blink_leds(u8 st, u8 value[])
{
	u8 v = 0;

	if (st == SLOT_READY) {
		if (((value[0] & 0xf) == (value[1] & 0xf)) &&
		    ((value[0] & 0xf) == (value[2] & 0xf)))
			v = 1;
	}
	WRITE_LEDS(v);
}

static u8 change_rotation_speed(void)
{
	u8 speed;

	switch (READ_RSWITCH) {
	case 0:
		speed = 0x3f;
		break;
	case 1:
		speed = 0x1f;
		break;
	default:
		speed = 0x0f;
		break;
	}

	return speed;
}

static int get_number_pushed_switches(u8 st)
{
	int num = 0;
	int i;

	for (i = 0; i < SWITCH_NUM; i++) {
		if ((st >> i) & 0x1)
			num++;
	}

	return num;
}

int slot(void)
{
	static u8 schedule = 0;
	static u8 sw_status = 0;
	static u8 slot_status = SLOT_READY;
	static u8 seg7_status = 7;	   /* 0: unpushed, 1: pushed */
	static u8 seg7_val[SEG7_NUM] = {3,2,1};
	int	 i;

	schedule++;

	sw_status = clear_switch_chattering();

	switch (slot_status) {
	case SLOT_READY:
		if (get_number_pushed_switches(sw_status) > 1)
			slot_status = SLOT_RUN;
		break;
	case SLOT_RUN:
		if (sw_status == 0)
			slot_status = SLOT_STOP;
		break;
	case SLOT_STOP:
		if (seg7_status == SEG7_ALL)
			slot_status = SLOT_READY;
		break;
	}

	if (slot_status == SLOT_STOP) {
		for (i = 0; i < SWITCH_NUM; i++) {
			if (sw_status & SWITCH(i))
				seg7_status |= SEG7(i);
		}
	} else if (slot_status == SLOT_RUN)
		seg7_status = 0;

	if ((schedule & change_rotation_speed()) == change_rotation_speed()) {
		for (i = 0; i < SEG7_NUM; i++) {
			if ((seg7_status & SEG7(i)) == 0)
				seg7_val[i]++;
		}
	}

	for (i = 0; i < SEG7_NUM; i++)
		WRITE_SEG7(i, seg7_val[i]);

	blink_leds(slot_status, seg7_val);

	return 0;
}
