/*
 * srec.c
 *
 * S-Record Handler
 *
 * Copyright (C) 2004-2008 Atmark Techno, Inc.	All Rights Reserved.
 *
 */

#include <ctype.h>

#include "srec.h"
#include "spi.h"
#include "memmap.h"
#include "flash.h"

static unsigned char nybble_to_val(char x)
{
	if (isdigit(x))
		return x - '0';
	else if (isupper(x))
		return x - 'A' + 10;
	else if (islower(x))
		return x - 'a' + 10;
	else
		return 0;
}

static int grab_hex_byte(char *buffer)
{
	return nybble_to_val(buffer[0])*16 + nybble_to_val(buffer[1]);
}

static unsigned int grab_hex_word(char *buffer)
{
	return grab_hex_byte(buffer)*256 + grab_hex_byte(buffer+2);
}

static unsigned int grab_hex_word3(char *buffer)
{
	return grab_hex_byte(buffer)*65536 + grab_hex_byte(buffer+2)*256 +
		grab_hex_byte(buffer+4);
}

static unsigned int grab_hex_dword(char *buffer)
{
	return grab_hex_word(buffer)*65536 + grab_hex_word(buffer+4);
}

static unsigned short convert_to_unsigned_short(int a, int b)
{
	return (unsigned short)(((a << 8) & 0xff00) | (b & 0x00ff));
}

int srec_decode_line(char *buffer)
{
	static int at_first = 1;		/* flag for at first */
	static unsigned long start_addr;	/* start address */
	static unsigned long last_addr;		/* last address */
	int type, data_count, count;
	unsigned long addr = 0;			/* to shut gcc up */
	char *data_ptr = 0;			/* as well */
	unsigned char cksum = 0;
	char *buf;
	int i;

	if (buffer[0] != 'S')
		return SREC_DOWNLOAD_CONTINUE;

	type = buffer[1]-'0';
	count = grab_hex_byte(buffer+2);
	data_count = 0;

	buf = buffer+2;
	for (i = 0; i < (count+1); i++) {
		cksum += grab_hex_byte(buf);
		buf += 2;
	}
	if (cksum != 0xff)
		return SREC_CHECKSUM_MISMATCH;

	switch (type) {
	case 1:
		addr = grab_hex_word(buffer+4);
		data_ptr = buffer+8;
		data_count = count-3;
		break;
	case 3:
		addr = grab_hex_dword(buffer+4);
		data_ptr = buffer+12;
		data_count = count-5;
		break;
	case 2:
		addr = grab_hex_word3(buffer+4);
		data_ptr = buffer+10;
		data_count = count-4;
		break;
	case 0:	/* START */
		at_first = 1;
		break;
	case 7:	/* END */
	case 8:
	case 9:
#ifdef SPI_REGISTER_BASEADDR
		spi_copy_from_dram(start_addr, last_addr);
#else
		flash_copy_from_dram(start_addr, last_addr);
#endif
		return SREC_DOWNLOAD_FINISH;
	default:
		return SREC_DOWNLOAD_CONTINUE;
	}

	/* Write the received data to sdram(at first) */
	if (data_count) {
		if (at_first) {
			at_first = 0;
			start_addr = addr;
			last_addr = start_addr;
		}
	}

	while (data_count > 0) {
		int a, b;
		unsigned short val;

		a = grab_hex_byte(data_ptr);
		data_ptr += 2;
		b = grab_hex_byte(data_ptr);
		data_ptr += 2;

		val = convert_to_unsigned_short(a, b);

		/* Write the received data to SDRAM */
		*((volatile unsigned short *)
		  (SDRAM_BASEADDR | (addr & 0x7FFFFF))) = val;

		addr += 2;	 /* for 2 byte(short) */
		data_count -= 2; /* as well */
	}

	/* Update the last address */
	last_addr = addr - 1;

	return SREC_DOWNLOAD_CONTINUE;
}
