------------------------------------------------------------------------------
-- user_logic.vhd - entity/architecture pair
------------------------------------------------------------------------------
--
-- ***************************************************************************
-- ** Copyright (c) 1995-2009 Xilinx, Inc.  All rights reserved.            **
-- **                                                                       **
-- ** Xilinx, Inc.                                                          **
-- ** XILINX IS PROVIDING THIS DESIGN, CODE, OR INFORMATION "AS IS"         **
-- ** AS A COURTESY TO YOU, SOLELY FOR USE IN DEVELOPING PROGRAMS AND       **
-- ** SOLUTIONS FOR XILINX DEVICES.  BY PROVIDING THIS DESIGN, CODE,        **
-- ** OR INFORMATION AS ONE POSSIBLE IMPLEMENTATION OF THIS FEATURE,        **
-- ** APPLICATION OR STANDARD, XILINX IS MAKING NO REPRESENTATION           **
-- ** THAT THIS IMPLEMENTATION IS FREE FROM ANY CLAIMS OF INFRINGEMENT,     **
-- ** AND YOU ARE RESPONSIBLE FOR OBTAINING ANY RIGHTS YOU MAY REQUIRE      **
-- ** FOR YOUR IMPLEMENTATION.  XILINX EXPRESSLY DISCLAIMS ANY              **
-- ** WARRANTY WHATSOEVER WITH RESPECT TO THE ADEQUACY OF THE               **
-- ** IMPLEMENTATION, INCLUDING BUT NOT LIMITED TO ANY WARRANTIES OR        **
-- ** REPRESENTATIONS THAT THIS IMPLEMENTATION IS FREE FROM CLAIMS OF       **
-- ** INFRINGEMENT, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS       **
-- ** FOR A PARTICULAR PURPOSE.                                             **
-- **                                                                       **
-- ***************************************************************************
--
------------------------------------------------------------------------------
-- Filename:          user_logic.vhd
-- Version:           1.00.b
-- Description:       User logic.
-- Date:              Tue Jun 01 13:15:30 2010 (by Create and Import Peripheral Wizard)
-- VHDL Standard:     VHDL'93
------------------------------------------------------------------------------
-- Naming Conventions:
--   active low signals:                    "*_n"
--   clock signals:                         "clk", "clk_div#", "clk_#x"
--   reset signals:                         "rst", "rst_n"
--   generics:                              "C_*"
--   user defined types:                    "*_TYPE"
--   state machine next state:              "*_ns"
--   state machine current state:           "*_cs"
--   combinatorial signals:                 "*_com"
--   pipelined or register delay signals:   "*_d#"
--   counter signals:                       "*cnt*"
--   clock enable signals:                  "*_ce"
--   internal version of output port:       "*_i"
--   device pins:                           "*_pin"
--   ports:                                 "- Names begin with Uppercase"
--   processes:                             "*_PROCESS"
--   component instantiations:              "<ENTITY_>I_<#|FUNC>"
------------------------------------------------------------------------------

-- DO NOT EDIT BELOW THIS LINE --------------------
library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_unsigned.all;

library proc_common_v3_00_a;
use proc_common_v3_00_a.proc_common_pkg.all;

-- DO NOT EDIT ABOVE THIS LINE --------------------

--USER libraries added here

------------------------------------------------------------------------------
-- Entity section
------------------------------------------------------------------------------
-- Definition of Generics:
--   C_SLV_AWIDTH                 -- Slave interface address bus width
--   C_SLV_DWIDTH                 -- Slave interface data bus width
--   C_NUM_MEM                    -- Number of memory spaces
--
-- Definition of Ports:
--   Bus2IP_Clk                   -- Bus to IP clock
--   Bus2IP_Reset                 -- Bus to IP reset
--   Bus2IP_Addr                  -- Bus to IP address bus
--   Bus2IP_CS                    -- Bus to IP chip select for user logic memory selection
--   Bus2IP_RNW                   -- Bus to IP read/not write
--   Bus2IP_Data                  -- Bus to IP data bus
--   Bus2IP_BE                    -- Bus to IP byte enables
--   IP2Bus_Data                  -- IP to Bus data bus
--   IP2Bus_RdAck                 -- IP to Bus read transfer acknowledgement
--   IP2Bus_WrAck                 -- IP to Bus write transfer acknowledgement
--   IP2Bus_Error                 -- IP to Bus error response
------------------------------------------------------------------------------

entity user_logic is
  generic
  (
    -- ADD USER GENERICS BELOW THIS LINE ---------------
    --USER generics added here
    -- ADD USER GENERICS ABOVE THIS LINE ---------------

    -- DO NOT EDIT BELOW THIS LINE ---------------------
    -- Bus protocol parameters, do not add to or delete
    C_SLV_AWIDTH                   : integer              := 32;
    C_SLV_DWIDTH                   : integer              := 32;
    C_NUM_MEM                      : integer              := 1
    -- DO NOT EDIT ABOVE THIS LINE ---------------------
  );
  port
  (
    -- ADD USER PORTS BELOW THIS LINE ------------------
    --USER ports added here
    -- ADD USER PORTS ABOVE THIS LINE ------------------
    SPI_SCK                        : out std_logic_vector(0 to 1);
    SPI_MISO                       : in  std_logic_vector(0 to 1);
    SPI_MOSI                       : out std_logic_vector(0 to 1);
    SPI_SS                         : out std_logic_vector(0 to 1);
    -- DO NOT EDIT BELOW THIS LINE ---------------------
    -- Bus protocol ports, do not add to or delete
    Bus2IP_Clk                     : in  std_logic;
    Bus2IP_Reset                   : in  std_logic;
    Bus2IP_Addr                    : in  std_logic_vector(0 to C_SLV_AWIDTH-1);
    Bus2IP_CS                      : in  std_logic_vector(0 to C_NUM_MEM-1);
    Bus2IP_RNW                     : in  std_logic;
    Bus2IP_Data                    : in  std_logic_vector(0 to C_SLV_DWIDTH-1);
    Bus2IP_BE                      : in  std_logic_vector(0 to C_SLV_DWIDTH/8-1);
    IP2Bus_Data                    : out std_logic_vector(0 to C_SLV_DWIDTH-1);
    IP2Bus_RdAck                   : out std_logic;
    IP2Bus_WrAck                   : out std_logic;
    IP2Bus_Error                   : out std_logic
    -- DO NOT EDIT ABOVE THIS LINE ---------------------
  );

  attribute SIGIS : string;
  attribute SIGIS of Bus2IP_Clk    : signal is "CLK";
  attribute SIGIS of Bus2IP_Reset  : signal is "RST";

end entity user_logic;

------------------------------------------------------------------------------
-- Architecture section
------------------------------------------------------------------------------

architecture imp of user_logic is

  --USER signal declarations added here, as needed for user logic
  component spi_core is
    port (
      SYS_Rst            : in  std_logic;
      SYS_CLK            : in  std_logic;
      SPI_SS             : in  std_logic_vector(0 to 1);
      SPI_MISO           : in  std_logic_vector(0 to 1);
      SPI_MOSI           : out std_logic_vector(0 to 1);
      SPI_CLK            : out std_logic_vector(0 to 1);
      tx_fifo_raed_en    : out std_logic;
      tx_fifo_valid      : in  std_logic;
      tx_fifo_read_data  : in  std_logic_vector(0 to 7);
      rx_fifo_write_en   : out std_logic;
      rx_fifo_write_data : out std_logic_vector(0 to 7);
      loop_mode          : in  std_logic
    );
  end component ;

  component fifo_16
    port (
      clk                : in std_logic;
      din                : in std_logic_vector(7 downto 0);
      rd_en              : in std_logic;
      rst                : in std_logic;
      wr_en              : in std_logic;
      dout               : out std_logic_vector(7 downto 0);
      empty              : out std_logic;
      full               : out std_logic
    );
  end component;

  ------------------------------------------
  -- Signals for user logic memory space example
  ------------------------------------------
  type BYTE_RAM_TYPE is array (0 to 255) of std_logic_vector(0 to 7);
  type DO_TYPE is array (0 to C_NUM_MEM-1) of std_logic_vector(0 to C_SLV_DWIDTH-1);
  signal mem_address               : std_logic_vector(0 to 7);
  signal mem_read_enable           : std_logic;
  signal mem_write_enable          : std_logic;
  signal mem_ip2bus_data           : std_logic_vector(0 to C_SLV_DWIDTH-1);
  signal mem_read_ack_dly1         : std_logic;
  signal mem_read_ack_dly2         : std_logic;
  signal mem_read_ack              : std_logic;
  signal mem_write_ack             : std_logic;
  signal spe                       : std_logic;
  signal ss0                       : std_logic;
  signal ss1                       : std_logic;
  signal loop_mode                 : std_logic;
  signal master                    : std_logic;
  signal cpol                      : std_logic;
  signal cpha                      : std_logic;
  signal man_ssaen                 : std_logic;
  signal mst_tranen                : std_logic;
  signal tx_fifo_valid             : std_logic;
  signal tx_fifo_reset             : std_logic;
  signal tx_fifo_data              : std_logic_vector(0 to 7);
  signal tx_fifo_write_en          : std_logic;
  signal tx_fifo_read_data         : std_logic_vector(0 to 7);
  signal tx_fifo_raed_en           : std_logic;
  signal tx_fifo_full              : std_logic;
  signal tx_fifo_empty             : std_logic;
  signal rx_fifo_reset             : std_logic;
  signal rx_fifo_write_data        : std_logic_vector(0 to 7);
  signal rx_fifo_write_en          : std_logic;
  signal rx_fifo_data              : std_logic_vector(0 to 7);
  signal rx_fifo_read_en           : std_logic;
  signal rx_fifo_full              : std_logic;
  signal rx_fifo_empty             : std_logic;

begin

  --USER logic implementation added here

  mem_read_enable <= ( Bus2IP_CS(0) ) and Bus2IP_RNW;
  mem_write_enable   <= ( Bus2IP_CS(0) ) and not(Bus2IP_RNW);
  mem_address     <= Bus2IP_Addr(C_SLV_AWIDTH-8 to C_SLV_AWIDTH-1);

------------------------------------------------
-- Read
------------------------------------------------

  process (Bus2IP_Clk) begin
    if Bus2IP_Clk'event and Bus2IP_Clk = '1' then
      if Bus2IP_Reset = '1' then
        mem_read_ack_dly1 <= '0';
        mem_read_ack_dly2 <= '0';
        mem_read_ack <= '0';
      else
        if mem_read_enable = '1' and mem_read_ack = '0' then
          mem_read_ack_dly1 <= mem_read_enable;
          mem_read_ack_dly2 <= mem_read_ack_dly1;
          mem_read_ack <= mem_read_ack_dly2;
        else
          mem_read_ack_dly1 <= '0';
          mem_read_ack_dly2 <= '0';
          mem_read_ack <= '0';
        end if;
      end if;
    end if;
  end process;


  process (Bus2IP_Clk) begin
    if Bus2IP_Clk'event and Bus2IP_Clk = '1' then
      if Bus2IP_Reset = '1' then
        rx_fifo_read_en <= '0';
      else
        if mem_address = x"6c" and mem_read_enable = '1' and mem_read_ack_dly1 = '0' then
          rx_fifo_read_en <= '1';
        else
         rx_fifo_read_en <= '0';
        end if;
      end if;
    end if;
  end process;


  process (Bus2IP_Clk) begin
    if Bus2IP_Clk'event and Bus2IP_Clk = '1' then
      if Bus2IP_Reset = '1' then
        IP2Bus_Data <= (others => '0');
      else
        if mem_read_ack_dly2 = '1' and mem_read_ack = '0' then
          case mem_address is
            when x"60" =>
              IP2Bus_Data(C_SLV_DWIDTH-1) <= loop_mode;
              IP2Bus_Data(C_SLV_DWIDTH-2) <= spe;
              IP2Bus_Data(C_SLV_DWIDTH-3) <= master;
              IP2Bus_Data(C_SLV_DWIDTH-4) <= cpol;
              IP2Bus_Data(C_SLV_DWIDTH-5) <= cpha;
              IP2Bus_Data(C_SLV_DWIDTH-6) <= tx_fifo_reset;
              IP2Bus_Data(C_SLV_DWIDTH-7) <= rx_fifo_reset;
              IP2Bus_Data(C_SLV_DWIDTH-8) <= man_ssaen ;
              IP2Bus_Data(C_SLV_DWIDTH-9) <= mst_tranen;
              IP2Bus_Data(0 to C_SLV_DWIDTH-1) <= (others => '0');
            when x"64" =>
              IP2Bus_Data(C_SLV_DWIDTH-1) <= rx_fifo_empty;
              IP2Bus_Data(C_SLV_DWIDTH-2) <= rx_fifo_full;
              IP2Bus_Data(C_SLV_DWIDTH-3) <= tx_fifo_empty;
              IP2Bus_Data(C_SLV_DWIDTH-4) <= tx_fifo_full;
              IP2Bus_Data(0 to C_SLV_DWIDTH-5) <= (others => '0');
            when x"6c" =>
              IP2Bus_Data(C_SLV_DWIDTH-8 to C_SLV_DWIDTH-1) <= rx_fifo_data;
              IP2Bus_Data(0 to C_SLV_DWIDTH-9) <= (others => '0');
            when x"70" =>
              IP2Bus_Data(C_SLV_DWIDTH-1) <= ss1;
              IP2Bus_Data(C_SLV_DWIDTH-2) <= ss0;
              IP2Bus_Data(0 to C_SLV_DWIDTH-3) <= (others => '0');
            when others =>
              IP2Bus_Data <= (others => '0');
          end case;
        else
          IP2Bus_Data <= (others => '0');
        end if;
      end if;
    end if;
  end process;

------------------------------------------------
-- Write
------------------------------------------------

  process (Bus2IP_Clk) begin
    if Bus2IP_Clk'event and Bus2IP_Clk = '1' then
      if Bus2IP_Reset = '1' then
        mem_write_ack <= '0';
      else
        if mem_write_enable = '1' and mem_write_ack = '0' then
          mem_write_ack <= mem_write_enable;
        else
          mem_write_ack <= '0';
        end if;
      end if;
    end if;
  end process;

  process (Bus2IP_Clk) begin
    if Bus2IP_Clk'event and Bus2IP_Clk = '1' then
      if Bus2IP_Reset = '1' then
        tx_fifo_write_en <= '0';
      else
        if mem_address = x"68" and mem_write_enable = '1' and mem_write_ack = '0' then
          tx_fifo_write_en <= '1';
        else
          tx_fifo_write_en <= '0';
        end if;
      end if;
    end if;
  end process;

  tx_fifo_data <= Bus2IP_Data(C_SLV_DWIDTH-8 to C_SLV_DWIDTH-1);

  process (Bus2IP_Clk) begin
    if Bus2IP_Clk'event and Bus2IP_Clk = '1' then
      if Bus2IP_Reset = '1' then
        loop_mode     <= '0';
        spe           <= '0';
        master        <= '0';
        cpol          <= '0';
        cpha          <= '0';
        tx_fifo_reset <= '0';
        rx_fifo_reset <= '0';
        man_ssaen     <= '1';
        mst_tranen    <= '1';
        ss0           <= '1';
        ss1           <= '1';
      else
        if mem_write_ack = '1' then
          case mem_address is
            when x"60" =>
              loop_mode     <= Bus2IP_Data(C_SLV_DWIDTH-1);
              spe           <= Bus2IP_Data(C_SLV_DWIDTH-2);
              master        <= Bus2IP_Data(C_SLV_DWIDTH-3);
              cpol          <= Bus2IP_Data(C_SLV_DWIDTH-4);
              cpha          <= Bus2IP_Data(C_SLV_DWIDTH-5);
              tx_fifo_reset <= Bus2IP_Data(C_SLV_DWIDTH-6);
              rx_fifo_reset <= Bus2IP_Data(C_SLV_DWIDTH-7);
              man_ssaen     <= Bus2IP_Data(C_SLV_DWIDTH-8);
              mst_tranen    <= Bus2IP_Data(C_SLV_DWIDTH-9);
            when x"70" =>
              ss1 <= Bus2IP_Data(C_SLV_DWIDTH-1);
              ss0 <= Bus2IP_Data(C_SLV_DWIDTH-2);
            when others =>
          end case;
        else
          tx_fifo_reset <= '0' or not spe;
          rx_fifo_reset <= '0' or not spe;
        end if;
      end if;
    end if;
  end process;

  SPI_SS(0) <= ss0;
  SPI_SS(1) <= ss1;


  IP2Bus_WrAck <= mem_write_ack;
  IP2Bus_RdAck <= mem_read_ack;
  IP2Bus_Error <= '0';


  ------------------------------------------
  -- SPI Core
  ------------------------------------------

  tx_fifo_valid <= not tx_fifo_empty;

  spi_core_0 : spi_core
    port map (
      SYS_CLK             => Bus2IP_Clk,
      SYS_RST             => Bus2IP_Reset,
      SPI_SS(0)           => ss0,
      SPI_SS(1)           => ss1,
      SPI_MISO            => SPI_MISO,
      SPI_MOSI            => SPI_MOSI,
      SPI_CLK             => SPI_SCK,
      tx_fifo_raed_en     => tx_fifo_raed_en,
      tx_fifo_valid       => tx_fifo_valid,
      rx_fifo_write_en    => rx_fifo_write_en,
      tx_fifo_read_data   => tx_fifo_read_data,
      rx_fifo_write_data  => rx_fifo_write_data,
      loop_mode           => loop_mode
    );


  tx_fifo_16 : fifo_16
    port map (
      clk                 => Bus2IP_Clk,
      rst                 => tx_fifo_reset,
      din                 => tx_fifo_data,
      wr_en               => tx_fifo_write_en,
      dout                => tx_fifo_read_data,
      rd_en               => tx_fifo_raed_en,
      empty               => tx_fifo_empty,
      full                => tx_fifo_full
    );


  rx_fifo_16 : fifo_16
    port map (
      clk                 => Bus2IP_Clk,
      rst                 => rx_fifo_reset,
      din                 => rx_fifo_write_data,
      wr_en               => rx_fifo_write_en,
      dout                => rx_fifo_data,
      rd_en               => rx_fifo_read_en,
      empty               => rx_fifo_empty,
      full                => rx_fifo_full
    );


end imp;
